@@ -45,21 +45,22 @@ PyObject *OSAObj_New(ComponentInstance itself)
 {
 	OSAComponentInstanceObject *it;
 	if (itself == NULL) {
-						PyErr_SetString(OSA_Error,"NULL ComponentInstance");
-						return NULL;
-					}
+	                                PyErr_SetString(OSA_Error,"NULL ComponentInstance");
+	                                return NULL;
+	                        }
 	it = PyObject_NEW(OSAComponentInstanceObject, &OSAComponentInstance_Type);
 	if (it == NULL) return NULL;
 	it->ob_itself = itself;
 	return (PyObject *)it;
 }
+
 int OSAObj_Convert(PyObject *v, ComponentInstance *p_itself)
 {
 
-				if (CmpInstObj_Convert(v, p_itself))
-					return 1;
-				PyErr_Clear();
-				
+	                if (CmpInstObj_Convert(v, p_itself))
+	                        return 1;
+	                PyErr_Clear();
+	                
 	if (!OSAObj_Check(v))
 	{
 		PyErr_SetString(PyExc_TypeError, "OSAComponentInstance required");
@@ -725,6 +726,7 @@ static PyObject *OSAObj_OSAMakeContext(OSAComponentInstanceObject *_self, PyObje
 	return _res;
 }
 
+#ifdef HAVE_OSA_DEBUG
 static PyObject *OSAObj_OSADebuggerCreateSession(OSAComponentInstanceObject *_self, PyObject *_args)
 {
 	PyObject *_res = NULL;
@@ -1033,6 +1035,8 @@ static PyObject *OSAObj_OSADebuggerDisposeCallFrame(OSAComponentInstanceObject *
 	return _res;
 }
 
+#endif /* HAVE_OSA_DEBUG */
+
 static PyMethodDef OSAObj_methods[] = {
 	{"OSALoad", (PyCFunction)OSAObj_OSALoad, 1,
 	 PyDoc_STR("(AEDesc scriptData, long modeFlags) -> (OSAID resultingScriptID)")},
@@ -1090,6 +1094,7 @@ static PyMethodDef OSAObj_methods[] = {
 	 PyDoc_STR("(AppleEvent theAppleEvent, OSAID contextID, long modeFlags) -> (AppleEvent reply)")},
 	{"OSAMakeContext", (PyCFunction)OSAObj_OSAMakeContext, 1,
 	 PyDoc_STR("(AEDesc contextName, OSAID parentContext) -> (OSAID resultingContextID)")},
+#ifdef HAVE_OSA_DEBUG
 	{"OSADebuggerCreateSession", (PyCFunction)OSAObj_OSADebuggerCreateSession, 1,
 	 PyDoc_STR("(OSAID inScript, OSAID inContext) -> (OSADebugSessionRef outSession)")},
 	{"OSADebuggerGetSessionState", (PyCFunction)OSAObj_OSADebuggerGetSessionState, 1,
@@ -1118,6 +1123,7 @@ static PyMethodDef OSAObj_methods[] = {
 	 PyDoc_STR("(OSADebugCallFrameRef inCurrentFrame) -> (OSADebugCallFrameRef outPrevFrame)")},
 	{"OSADebuggerDisposeCallFrame", (PyCFunction)OSAObj_OSADebuggerDisposeCallFrame, 1,
 	 PyDoc_STR("(OSADebugCallFrameRef inCallFrame) -> None")},
+#endif /* HAVE_OSA_DEBUG */
 	{NULL, NULL, 0}
 };
 
@@ -1133,16 +1139,16 @@ static PyMethodDef OSAObj_methods[] = {
 
 #define OSAObj_tp_alloc PyType_GenericAlloc
 
-static PyObject *OSAObj_tp_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
+static PyObject *OSAObj_tp_new(PyTypeObject *type, PyObject *_args, PyObject *_kwds)
 {
-	PyObject *self;
+	PyObject *_self;
 	ComponentInstance itself;
 	char *kw[] = {"itself", 0};
 
-	if (!PyArg_ParseTupleAndKeywords(args, kwds, "O&", kw, OSAObj_Convert, &itself)) return NULL;
-	if ((self = type->tp_alloc(type, 0)) == NULL) return NULL;
-	((OSAComponentInstanceObject *)self)->ob_itself = itself;
-	return self;
+	if (!PyArg_ParseTupleAndKeywords(_args, _kwds, "O&", kw, OSAObj_Convert, &itself)) return NULL;
+	if ((_self = type->tp_alloc(type, 0)) == NULL) return NULL;
+	((OSAComponentInstanceObject *)_self)->ob_itself = itself;
+	return _self;
 }
 
 #define OSAObj_tp_free PyObject_Del
@@ -1210,8 +1216,8 @@ void init_OSA(void)
 
 
 	/*
-		PyMac_INIT_TOOLBOX_OBJECT_NEW(ComponentInstance, OSAObj_New);
-		PyMac_INIT_TOOLBOX_OBJECT_CONVERT(ComponentInstance, OSAObj_Convert);
+	        PyMac_INIT_TOOLBOX_OBJECT_NEW(ComponentInstance, OSAObj_New);
+	        PyMac_INIT_TOOLBOX_OBJECT_CONVERT(ComponentInstance, OSAObj_Convert);
 	*/
 
 
